@@ -13,6 +13,7 @@ import base64
 import hashlib
 from abc import ABC
 from typing import Any, List, Optional, Tuple, Dict, Iterable, Union, Callable
+from contextlib import suppress
 from . import Error, Type, Env
 
 
@@ -185,9 +186,8 @@ class String(Base):
             if isinstance(desired_type, Type.Float):
                 return Float(float(self.value), self.expr)
         except ValueError as exn:
-            if self.expr:
-                raise Error.EvalError(self.expr, "coercing String to number: " + str(exn)) from exn
-            raise
+            msg = f"coercing String to {desired_type}: {exn}"
+            raise Error.EvalError(self.expr, msg) if self.expr else Error.RuntimeError(msg)
         return super().coerce(desired_type)
 
 
@@ -325,13 +325,16 @@ class Map(Base):
             )
         if isinstance(desired_type, Type.StructInstance):
             if self.type.item_type[0] == Type.String():
-                # Runtime typecheck for initializing struct from read_{object[s],map,json}
+                # Runtime typecheck for initializing struct from read_{object,objects,map}
                 # This couldn't have been checked statically because the map keys weren't known.
                 litty = Type.Map(
                     self.type.item_type, self.type.optional, set(kv[0].value for kv in self.value)
                 )
                 if not litty.coerces(desired_type):
-                    msg = "runtime struct initializer doesn't have the required fields with the expected types"
+                    msg = (
+                        "unusable runtime struct initializer"
+                        " (member type mismatch, lacking required member, or extra member)"
+                    )
                     raise Error.EvalError(
                         self.expr,
                         msg,
@@ -340,8 +343,17 @@ class Map(Base):
             ans = {}
             for k, v in self.value:
                 k = k.coerce(Type.String()).value
-                assert k in desired_type.members
-                ans[k] = v
+                try:
+                    ans[k] = v.coerce(desired_type.members[k])
+                except Error.RuntimeError as exc:
+                    msg = (
+                        "runtime type mismatch initializing struct member "
+                        f"{str(desired_type.members[k])} {k}"
+                    )
+                    raise Error.EvalError(
+                        self.expr,
+                        msg,
+                    ) if self.expr else Error.RuntimeError(msg)
             return Struct(desired_type, ans, self.expr)
         return super().coerce(desired_type)
 
@@ -395,7 +407,7 @@ class Null(Base):
     ``type`` and ``value`` are both None."""
 
     def __init__(self, expr: "Optional[Expr.Base]" = None) -> None:
-        super().__init__(Type.Any(optional=True), None, expr)
+        super().__init__(Type.Any(null=True), None, expr)
 
     def coerce(self, desired_type: Optional[Type.Base] = None) -> Base:
         """"""
@@ -430,26 +442,86 @@ class Struct(Base):
         expr: "Optional[Expr.Base]" = None,
     ) -> None:
         value = dict(value)
-        if isinstance(type, Type.StructInstance):
-            assert type.members
+        if isinstance(type, (Type.Object, Type.StructInstance)):
+            type_members = type.members
+            assert type_members
             # coerce values to member types
             for k in value:
-                assert k in type.members
-                value[k] = value[k].coerce(type.members[k])
-            # if initializer (map or object literal) omits optional members,
-            # fill them in with null
-            for k in type.members:
+                try:
+                    value[k] = value[k].coerce(type_members[k])
+                except Error.RuntimeError:
+                    msg = (
+                        f"runtime type mismatch initializing struct member"
+                        f" {str(type_members[k])} {k}"
+                    )
+                    raise Error.EvalError(
+                        expr,
+                        msg,
+                    ) if expr else Error.RuntimeError(msg)
+            # if initializer omits optional members, fill them in with null
+            for k in type_members:
                 if k not in value:
-                    assert type.members[k].optional
+                    assert type_members[k].optional
                     value[k] = Null()
         self.value = value
         super().__init__(type, value, expr)
 
     def coerce(self, desired_type: Optional[Type.Base] = None) -> Base:
         """"""
-        if isinstance(self.type, Type.Object) and isinstance(desired_type, Type.StructInstance):
+        if isinstance(desired_type, (Type.Object, Type.StructInstance)):
+            if not self.type.coerces(desired_type):
+                msg = (
+                    "unusable runtime struct initializer"
+                    " (member type mismatch, lacking required member, or extra member)"
+                )
+                raise Error.EvalError(
+                    self.expr,
+                    msg,
+                ) if self.expr else Error.RuntimeError(msg)
             return Struct(desired_type, self.value, self.expr)
-        return self
+        if isinstance(desired_type, Type.Map):
+            return self._coerce_to_map(desired_type)
+        if isinstance(desired_type, Type.Any):
+            return self
+        msg = f"cannot coerce struct to {desired_type}"
+        raise Error.EvalError(
+            self.expr,
+            msg,
+        ) if self.expr else Error.RuntimeError(msg)
+
+    def _coerce_to_map(self, desired_type: Type.Map) -> Map:
+        # runtime coercion e.g. Map[String,String] foo = read_json("foo.txt")
+        assert isinstance(self.type, Type.Object)
+
+        def fail(msg):
+            raise Error.EvalError(
+                self.expr,
+                msg,
+            ) if self.expr else Error.RuntimeError(msg)
+
+        key_type = desired_type.item_type[0]
+        if not Type.String().coerces(key_type):
+            fail(f"cannot coerce member names to {key_type} map keys")
+        value_type = desired_type.item_type[1]
+        entries = []
+        for k, v in self.value.items():
+            if not (isinstance(v, Null) and value_type.optional):
+                map_key = None
+                try:
+                    map_key = String(k).coerce(key_type)
+                except Error.RuntimeError:
+                    fail(f"cannot coerce member name {k} to {key_type} map key")
+                map_value = None
+                if self.type.members[k].coerces(value_type):
+                    with suppress(Error.RuntimeError):
+                        map_value = v.coerce(value_type)
+                if map_value is None:
+                    fail(
+                        "cannot coerce struct member"
+                        f" {self.type.members[k]} {k} to {value_type} map value"
+                    )
+                entries.append((map_key, map_value))
+        return Map(desired_type.item_type, entries)
 
     def __str__(self) -> Any:
         return "{" + ", ".join(f"{k}: {str(v)}" for k, v in self.value.items()) + "}"
@@ -552,9 +624,13 @@ def _infer_from_json(j: Any) -> Base:
         item_type = Type.unify([item.type for item in items])
         return Array(item_type, [item.coerce(item_type) for item in items])
     if isinstance(j, dict):
-        items = [(String(str(k)), _infer_from_json(j[k])) for k in j]
-        value_type = Type.unify([v.type for _, v in items])
-        return Map((Type.String(), value_type), [(k, v.coerce(value_type)) for k, v in items])
+        members = {}
+        member_types = {}
+        for k in j:
+            assert isinstance(k, str)
+            members[k] = _infer_from_json(j[k])
+            member_types[k] = members[k].type
+        return Struct(Type.Object(member_types), members)
     raise Error.InputError(f"couldn't construct value from: {json.dumps(j)}")
 
 
