@@ -1,3 +1,4 @@
+from math import exp
 import unittest
 import logging
 import tempfile
@@ -550,7 +551,7 @@ class TestStdLib(unittest.TestCase):
                 Array[String] my_array = read_json(stdout())
             }
         }
-        """, expected_exception=WDL.Error.InputError)
+        """, expected_exception=WDL.Error.EvalError)
 
         outputs = self._test_task(R"""
         version 1.0
@@ -577,6 +578,18 @@ class TestStdLib(unittest.TestCase):
         }
         """, expected_exception=WDL.Error.InputError)
 
+        self._test_task(R"""
+        version 1.0
+        task test {
+            command <<<
+                echo '{"foo":"bar"}'
+            >>>
+            output {
+                String baz = read_json(stdout())["baz"]
+            }
+        }
+        """, expected_exception=WDL.Error.OutOfBounds)
+
     def test_read_map_ints(self):
         outputs = self._test_task(R"""
         version 1.0
@@ -675,6 +688,205 @@ class TestStdLib(unittest.TestCase):
         self.assertEqual(outputs["alice"], alice)
         self.assertEqual(outputs["samplesheet2"], samplesheet2)
 
+    def test_issue524(self):
+        # additional cases for struct initialization from read_json(), motivated by issue #524
+
+        # explicit null value should be acceptable initializer for optional struct field
+        outp = self._test_task(R"""
+        version 1.0
+
+        struct MyStruct {
+            Int x
+            String? y
+        }
+
+        task mytask {
+            input {
+            }
+
+            command <<<
+                cat > data.json <<EOL
+                {
+                    "x": 123,
+                    "y": null
+                }
+                EOL
+            >>>
+
+            output {
+                MyStruct data = read_json("data.json")
+            }
+        }
+        """)
+        self.assertEqual(outp["data"], {"x": 123, "y": None})
+        # elaboration with a heterogeneous unification:
+        outp = self._test_task(R"""
+        version 1.0
+
+        struct MyStruct {
+            Float x
+            String? y
+            Array[Int?] z
+        }
+
+        task mytask {
+            input {
+            }
+
+            command <<<
+                cat > data.json <<EOL
+                {
+                    "x": 3.14159,
+                    "y": null,
+                    "z": [4,2,null]
+                }
+                EOL
+            >>>
+
+            output {
+                MyStruct data = read_json("data.json")
+            }
+        }
+        """)
+        self.assertEqual(outp["data"], {"x": 3.14159, "y": None, "z": [4,2,None]})
+        # unusable null
+        self._test_task(R"""
+        version 1.0
+
+        struct MyStruct {
+            Float x
+            String? y
+            Array[Int?] z
+        }
+
+        task mytask {
+            input {
+            }
+
+            command <<<
+                cat > data.json <<EOL
+                {
+                    "x": "bogus",
+                    "y": null,
+                    "z": [4,2,null]
+                }
+                EOL
+            >>>
+
+            output {
+                MyStruct data = read_json("data.json")
+            }
+        }
+        """, expected_exception=WDL.Error.EvalError)
+        # top-level null
+        outp = self._test_task(R"""
+        version 1.0
+
+        struct MyStruct {
+            Float x
+            String? y
+            Array[Int?] z
+        }
+
+        task mytask {
+            input {
+            }
+
+            command <<<
+                echo null > data.json
+            >>>
+
+            output {
+                MyStruct? data = read_json("data.json")
+            }
+        }
+        """)
+        self.assertEqual(outp, {"data": None})
+        # coercion failure -- required member missing
+        self._test_task(R"""
+        version 1.0
+
+        struct MyStruct {
+            Float x
+            String? y
+            Array[Int?] z
+        }
+
+        task mytask {
+            input {
+            }
+
+            command <<<
+                cat > data.json <<EOL
+                {
+                    "y": null,
+                    "z": [4,2,null]
+                }
+                EOL
+            >>>
+
+            output {
+                MyStruct data = read_json("data.json")
+            }
+        }
+        """, expected_exception=WDL.Error.EvalError)
+        # bad coercion to Map (key type)
+        self._test_task(R"""
+        version 1.0
+
+        struct MyStruct {
+            Float x
+            String? y
+            Array[Int?] z
+        }
+
+        task mytask {
+            input {
+            }
+
+            command <<<
+                cat > data.json <<EOL
+                {
+                    "x": 3.14159,
+                    "z": [4,2,null]
+                }
+                EOL
+            >>>
+
+            output {
+                Map[Float,String] data = read_json("data.json")
+            }
+        }
+        """, expected_exception=WDL.Error.EvalError)
+        # bad coercion to Map (value type)
+        self._test_task(R"""
+        version 1.0
+
+        struct MyStruct {
+            Float x
+            String? y
+            Array[Int?] z
+        }
+
+        task mytask {
+            input {
+            }
+
+            command <<<
+                cat > data.json <<EOL
+                {
+                    "x": 3.14159,
+                    "z": [4,2,null]
+                }
+                EOL
+            >>>
+
+            output {
+                Map[String,Float] data = read_json("data.json")
+            }
+        }
+        """, expected_exception=WDL.Error.EvalError)
+
     def test_bad_object(self):
         self._test_task(R"""
         version 1.0
